/*******************************************************************************
  USB Device Layer local Header File

  Company:
    Microchip Technology Inc.

  File Name:
    usb_device_local.h

  Summary:
    USB device layer local header file.

  Description:
    This header file contains private data types required for usb_device_layer.c.
    This header file must not be included in the application.
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
//DOM-IGNORE-END

#ifndef _USB_DEVICE_LOCAL_H
#define _USB_DEVICE_LOCAL_H


// *****************************************************************************
// *****************************************************************************
// Section: File includes
// *****************************************************************************
// ***************************************************************************** /


#include "usb/usb_device.h"
#include "osal/osal.h"

// *****************************************************************************
// *****************************************************************************
// Section: Version Numbers
// *****************************************************************************
// *****************************************************************************
/* Versioning of the driver */

// *****************************************************************************
/* USB Device Layer Version Macros

  Summary:
    USB Device Layer  version.

  Description:
    These constants provide USB Device Layer  version information. The driver
    version is:
    _USB_DEVICE_VERSION_MAJOR._USB_DEVICE_VERSION_MINOR._USB_DEVICE_VERSION_PATCH.
    It is represented in _USB_DEVICE_VERSION as:
    MAJOR *10000 + MINOR * 100 + PATCH, so as to allow comparisons.
    It is also represented in string format in _USB_DEVICE_VERSION_STR.
    _USB_DEVICE_TYPE provides the type of the release when the release is alpha
    or beta. The interfaces _USB_DEVICE_VersionGet() and
    _USB_DEVICE_VersionStrGet() provide interfaces to the access the version
    and the version string.

  Remarks:
    Modify the return value of _USB_DEVICE_VersionStrGet and the
    _USB_DEVICE_VERSION_MAJOR, _USB_DEVICE_VERSION_MINOR,
    _USB_DEVICE_VERSION_PATCH and _USB_DEVICE_VERSION_TYPE.
*/

#define _USB_DEVICE_VERSION_MAJOR         1
#define _USB_DEVICE_VERSION_MINOR         0
#define _USB_DEVICE_VERSION_PATCH         0
#define _USB_DEVICE_VERSION_TYPE
#define _USB_DEVICE_VERSION_STR           "1.00"

// *****************************************************************************
/* USB function driver structure

  Summary:
    A function driver has to expose standard APIs to device layer using 
    following structure.

  Description:
    All function drivers (including vendor specific ones) must provide callback
    functions to USB device layer in the format specified by this structure.
    The USB device layer calls these callback functions at the time of
    appropriate event.

  Remarks:
    Even the vendor specific function drivers must provide callback functions
    in this format.
    
*/

typedef struct  
{
    /* Initialize gets called by the Device layer when it recieves set
       configuration event. The device layer will initialize a function driver
       for every descriptor. Based on the descriptor type the function driver
       has to initialize itself. */ 

    void (*initializeByDescriptor)
    (
        SYS_MODULE_INDEX funcDriverIndex, 
        USB_DEVICE_HANDLE usbDeviceHandle,
        void* funcDriverInit, 
        uint8_t interfaceNumber, 
        uint8_t alternateSetting,
        uint8_t descriptorType, 
        uint8_t * pDescriptor
    );

    /* Deinitialize gets called when the device layer detects a device dettach,
       change in configuration or ob USB bus reset.*/

    void (*deInitialize)(SYS_MODULE_INDEX funcDriverIndex);

    /* This function will be called by the device layer when there is a interface specific
       setup packet request */
    
    void (*controlTransferNotification) 
    (
        SYS_MODULE_INDEX index,
        USB_DEVICE_EVENT controlEvent,
        USB_SETUP_PACKET * controlEventData
    );

    /* Function driver Tasks */
    void (*tasks)(SYS_MODULE_INDEX funcDriverIndex);

} USB_DEVICE_FUNCTION_DRIVER;

// *****************************************************************************
/* USB device layer function driver data

  Summary:
    This structure has members that has information about the function drivers.

  Description:
    This structure has members that has information about the function drivers.

  Remarks:
    None.
*/

typedef struct 
{
    /* First Interface number used by this 
     * instance of function driver */
    uint8_t interfaceNumber;

    uint8_t numberOfInterfaces;

    /* Function driver instance index */
    SYS_MODULE_INDEX funcDriverIndex;

    /* Pointer to a structure which contains 
     * function driver initialization data
     */
    void * funcDriverInit;

    /* Pointer to a standard structure that 
     * exposes function driver APIs to USB
     * device layer*/
    USB_DEVICE_FUNCTION_DRIVER *  driver;

} USB_DEVICE_FUNC_DRIVER_DATA;

// *****************************************************************************
/* USB Device Layer Get Status Request Response.

  Summary:
    This is the USB Device Get Status Request Response.

  Description:
    When the USB Device Layer gets a Get Status Request from the host, it
    responds with this data structure.

  Remarks:
    None.
*/

typedef union
{
    struct{
       
        /* Self powered */
        unsigned selfPowered:1;
        /* Remote wake up */
        unsigned remoteWakeup:1;
         /* Reserved bits */
        unsigned :6;
        unsigned :8 ;
    };

    struct{
        
        /* Endpoint halt */
        unsigned endPointHalt:1;
        /*Reserved bits */
        unsigned :7;
        /* Reserved bits */
        unsigned :8;
    };

    uint16_t status;

} USB_DEVICE_STATUS_RESPONSE;

// *****************************************************************************
/* USB Device Layer Control Transfer Structure

  Summary:
    This is the USB Device Layer Control Transfer Structure

  Description:
    This is the USB Device Layer Control Transfer Structure. It tracks the
    progress of the control transfer and allows the device layer or the
    application to handle the control transfer.

  Remarks:
    None.
*/

typedef struct 
{
    /* Control transfer state */
    bool  inProgress;
    uint8_t rxDataCount;
    uint8_t expectedRxDataCount;
    SYS_MODULE_INDEX handlerIndex;
    uint8_t * rxBuffer;

    /* Temporary pointer to function driver provided control transfer function.
       The setup packet will be forwarded to this function driver provided control
       transfer function */ 

    void (* handler)
    (   
         SYS_MODULE_INDEX handlerIndex,
         USB_DEVICE_EVENT controlTransferState,
         void * eventData
    );

} USB_DEVICE_CONTROL_TRANSFER_STRUCT;


// *****************************************************************************
/* USB Device Layer Task States

  Summary:
    USB Device Layer Task States.

  Description:
    This enumeration defines the possible USB Device Layer Task States

  Remarks:
    None.
*/

typedef enum
{
    /* USB Device Layer is trying to open the USBCD */
    USB_DEVICE_TASK_STATE_OPENING_USBCD,

    /* USB Device Layer is running the main task */
    USB_DEVICE_TASK_STATE_RUNNING    

} USB_DEVICE_TASK_STATE;

// *****************************************************************************
/* USB Device Layer Status structure

  Summary:
    A structure with usb device layer status and states.

  Description:
    This is a structure with usb device layer status and states.

  Remarks:
    None.
*/

typedef union
{
    struct
    {
        USB_DEVICE_POWER_STATE powerState:1;
        USB_SPEED usbSpeed:3;
        bool setAddressPending:1;
        bool remoteWakeupStatus:1;
        bool isSuspended:1;
        USB_DEVICE_STATE usbDeviceState:3;
        USB_DEVICE_STATE usbDevStatePriorSuspend:3;
        unsigned :19;
    };
} USB_DEVICE_STATUS;

// *****************************************************************************
/* USB Device Layer Endpoint Queue Size structure

  Summary:
    A structure with usb device layer Endpoint queue sizes

  Description:
    This structure is used to maintain queue size for endpoint read and write
    functions.

  Remarks:
    None.
*/
typedef struct
{
    /* Maxiumum Queue size for Endpoint Read. This queue size is applicable to
       all vendor Endpoints  */
    uint16_t qSizeMaxEpRead;

    /* Maximum Queue size for Endpoint Write. This queue size is applicable to
       all vendor Endpoints  */
    uint16_t qSizeMaxEpWrite;

    /* Current Queue size for Endpoint Write. This queue size is applicable to
       all vendor Endpoints  */
    uint16_t qSizeCurrentEpRead;

    /* Current Queue size for Endpoint Write. This queue size is applicable to
       all vendor Endpoints  */
    uint16_t qSizeCurrentEpWrite;

} USB_DEVICE_Q_SIZE_ENDPOINT;

// *****************************************************************************
/* USB Device Layer Instance Structure

  Summary:
    USB Device Layer Instance Structure.

  Description:
    This is the USB Device Layer Instance data structure.

  Remarks:
    None.
*/

typedef struct 
{
    /* USB device status of this instance */ 
    USB_DEVICE_STATUS usbDeviceStatusStruct;

    /* State of this instance */
    SYS_STATUS usbDeviceInstanceState;

    /* Pointer to master descriptor table */
    USB_DEVICE_MASTER_DESCRIPTOR * ptrMasterDescTable;

    /* Number of function driver instances registered */
    uint16_t registeredFuncDriverCount;

     /* Function table registered to this instance of the USB device layer */
    USB_DEVICE_FUNCTION_REGISTRATION_TABLE * registeredFuncDrivers;

     /* This instance index */
    SYS_MODULE_INDEX usbDevLayerIndex;
    
    /* USB Controller driver handle */
    DRV_HANDLE usbCDHandle;

    /* USB Controller Driver System Module Object */
    SYS_MODULE_OBJ usbCDSystemModuleObject;
   
    /* EP0 rx buffer*/
    uint8_t ep0RxBuffer[USB_DEVICE_EP0_BUFFER_SIZE];

    /* Tx IRP */
    USB_DEVICE_IRP irpEp0Tx;

    /* Rx IRP */
    USB_DEVICE_IRP irpEp0Rx;

    /* Device address */
    uint8_t deviceAddress;   

    /* Current active configuration */
    uint8_t activeConfiguration;

    /* Maximum configurations available for the selected speed*/
    uint8_t maxConfigs;

    /* Pointer to configuration descriptor group for the selected speed*/
    USB_DEVICE_CONFIGURATION_DESCRIPTORS_TABLE * configDescriptorsPtr;

    /* Pointer to active host selected descriptor */
    uint8_t * pActiveConfigDesc;

    /* Currently active events */
    USB_DEVICE_EVENT event;

    /* The two bytes that we are going to reply with for
       standard GET_STATUS request */
    USB_DEVICE_STATUS_RESPONSE getStatusResponse;

    /* Remote wakeup status as set by the host */
    USB_DEVICE_REMOTE_WAKEUP_STATUS remoteWakeupStatus;

    /* Control transfer related structure */
    USB_DEVICE_CONTROL_TRANSFER_STRUCT controlTransfer;

    /* Device Layer Tasks routine state */
    USB_DEVICE_TASK_STATE taskState;

    /* Client status */
    USB_DEVICE_CLIENT_STATUS clientState;

    /* Set to true if this object is in use */
    bool inUse;

    /* Pointer to the USB Device event callback function */
    USB_DEVICE_EVENT_HANDLER callBackFunc;

     /* Context associated with this client */
    uintptr_t context;
	
    /* This points to the size of the current control transfer */
    uint16_t controlTransferDataStageSize;
    
    /* Set to true if all members of this structure
       have been initialized once */
    bool isMutexEndpointIrpInitialized;

    /* Mutex to protect Endpoint IRP array */
    OSAL_MUTEX_DECLARE(mutexEndpointIRP);

    

} USB_DEVICE_OBJ;

// *****************************************************************************
/*  USB device layer Device descriptor header

  Summary:
    USB device layer Device descriptor header

  Description:
    USB device layer Device descriptor header

  Remarks:
   None.
*/

typedef struct __attribute__ ((packed))
{
    uint8_t bLength;
    uint8_t bDescriptorType;

} USB_DEVICE_SERVICE_DESCRIPTOR_HEAD;

/*******************************************************
 * Local functions
 *******************************************************/
void  _USB_DEVICE_ControlTransferHandler
(
    SYS_MODULE_INDEX handlerIndex,
    USB_DEVICE_EVENT transferEvent,
    void * eventData
);

void _USB_DEVICE_ProcessStandardDeviceSetRequests
(
    USB_DEVICE_OBJ * usbDeviceInstance,
    USB_SETUP_PACKET * setupPkt
);

void _USB_DEVICE_ProcessStandardDeviceGetRequests
(
    USB_DEVICE_OBJ * usbDeviceInstance,
    USB_SETUP_PACKET * setupPkt 
);

void _USB_DEVICE_ProcessStandardEndpointRequest
(
    USB_DEVICE_OBJ * usbDeviceInstance,
    uint8_t interfaceNumber,
    USB_SETUP_PACKET * setupPkt
);

void _USB_DEVICE_RedirectControlXfrToClient
(
    USB_DEVICE_OBJ* usbDeviceThisInstance ,
        USB_DEVICE_EVENT event, 
    USB_SETUP_PACKET * setupPkt
);
void _USB_DEVICE_ForwardControlXfrToFunction
(
    USB_DEVICE_OBJ * usbDeviceThisInstance,
    uint8_t interfaceNumber,
    USB_SETUP_PACKET * setupPkt
);
void _USB_DEVICE_ProcessInterfaceRequests
(
    USB_DEVICE_OBJ* usbDeviceInstance ,
    USB_SETUP_PACKET * setupPkt
);

void _USB_DEVICE_EventHandler
(
    uintptr_t referenceHandle,
    DRV_USB_EVENT eventType, 
    void * eventData 
);

USB_DEVICE_OBJ* _USB_DEVICE_ClientHandleValidate
(
    USB_DEVICE_HANDLE deviceHandle
);

USB_DEVICE_FUNCTION_REGISTRATION_TABLE * _USB_DEVICE_GetFunctionDriverEntryByInterface
(
    uint8_t interfaceNumber,
    USB_DEVICE_OBJ * usbDeviceThisInstance
);

bool _USB_DEVICE_FindEndpoint( USB_DEVICE_OBJ* usbDeviceThisInstance,
                          USB_ENDPOINT endpointNumber, uint8_t* interfaceNumber);
void _USB_DEVICE_Ep0ReceiveCompleteCallback( USB_DEVICE_IRP * handle );
void _USB_DEVICE_Ep0TransmitCompleteCallback(USB_DEVICE_IRP * handle);

void _USB_DEVICE_EndpointWriteCallBack( USB_DEVICE_IRP * irp );
void _USB_DEVICE_EndpointReadCallBack( USB_DEVICE_IRP * irp );
void _USB_DEVICE_RemotewakeupTimerCallback(uintptr_t context);
void _USB_DEVICE_Initialize_Endpoint_Q_Size(SYS_MODULE_INDEX index, uint16_t qSizeRead, uint16_t qSizeWrite ); 
#endif
